/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.classifier;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.ztest.classinfo.ZClassUtils;
import org.ztest.classinfo.ZIClassInfo;

public class ZClassifyByArchive implements ZIClassifier {
	private final ZIClassInfo classInfo;
	private final String suffix;

	public ZClassifyByArchive(ZIClassInfo classInfo, String suffix) {
		super();
		this.classInfo = classInfo;
		this.suffix = suffix;
	}

	public Set<String> classify(String name) throws Exception {
		Set<String> ret = new HashSet<String>();		
		List<String> locations = ZClassUtils.getLocationNames(classInfo, name); 
		for(String loc: locations)
		{
			if(loc.endsWith(suffix))
			{
				int idx = loc.lastIndexOf('/');
				if(idx>=0)
				{
					String arch = loc.substring(idx+1);
					idx = loc.lastIndexOf('\\');
					if(idx>=0)
					{						
						arch = loc.substring(idx+1);
					}
					ret.add(arch);
				}
			}
			else
			{
				ret.add(loc);
			}
		}
		return ret;
	}
}
