/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.classifier;

import java.util.HashSet;
import java.util.Set;

import org.ztest.classinfo.ZClassUtils;

public class ZClassifyByPackage implements ZIClassifier {

	private final int depth;

	public int getDepth() {
		return depth;
	}

	public ZClassifyByPackage(int depth) {
		super();
		this.depth = depth;
	}

	public ZClassifyByPackage() {
		this(Integer.MAX_VALUE);
	}

	public Set<String> classify(String name) throws Exception {
		Set<String> ret = new HashSet<String>();

		String packageName = ZClassUtils.getPackageName(name);
		while(packageName.startsWith("["))
		{
		    packageName = packageName.substring(1);
		}
		while(packageName.endsWith(";"))
		{
		    packageName = packageName.substring(1, packageName.length()-2);
		}
		
		int idx = packageName.indexOf('.');
		for (int i = 0; idx >= 0 && i < depth - 1; i++) {
			idx = packageName.indexOf('.', idx + 1);
		}
		if (idx >= 0) {
			String packageNameIdx = packageName.substring(0, idx);
			ret.add(packageNameIdx);
		} else {
			ret.add(packageName);
		}

		return ret;
	}
}
