package org.ztest.classinfo;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.jdom.Element;
import org.ztest.filter.ZCompoundClassEdgeFilter;
import org.ztest.filter.ZCompoundClassFilter;
import org.ztest.filter.ZFilterUtil;

public class ZClassInfoWrapper implements ZIClassInfo {
    private final ZIClassInfo classInfo;
    private final List<ZIClassFilter> classFilters = new ArrayList<ZIClassFilter>();
    private final List<ZIClassEdgeFilter> edgeFilters = new ArrayList<ZIClassEdgeFilter>();

    private final ZIClassFilter myClassFilter = new ZIClassFilter() {
        public boolean accept(String name)
            throws Exception {
            for (ZIClassFilter cf : classFilters) {
                if (!cf.accept(name)) {
                    return false;
                }
            }
            return true;
        }
    };

    private final ZIClassEdgeFilter myClassEdgeFilter = new ZIClassEdgeFilter() {
        public boolean accept(ZIClassEdge edge)
            throws Exception {
            if(!myClassFilter.accept(edge.getSource())) {
                return false;
            }
            if(!myClassFilter.accept(edge.getTarget())) {
                return false;
            }
            for (ZIClassEdgeFilter cf : edgeFilters) {
                if (!cf.accept(edge)) {
                    return false;
                }
            }
            return true;
        }
    };

    public ZClassInfoWrapper(ZIClassInfo classInfo) {
        super();
        this.classInfo = classInfo;
    }

    public List<ZIClassFilter> getClassFilters() {
        return classFilters;
    }

    public List<ZIClassEdgeFilter> getEdgeFilters() {
        return edgeFilters;
    }

    public void addClass(String name)
        throws Exception {
        classInfo.addClass(name);
    }

    public ZIClassEdge addEdge(String name1, String name2)
        throws Exception {
        return classInfo.addEdge(name1, name2);
    }

    public ZIClassEdge addEdgeAndNode(String name1, String name2)
        throws Exception {
        return classInfo.addEdgeAndNode(name1, name2);
    }

    public ZIClassMeta getClassMeta(String name)
        throws Exception {
        return classInfo.getClassMeta(name);
    }

    public Set<String> getClasses()
        throws Exception {
        return classInfo.getClasses(myClassFilter);
    }

    public Set<String> getClasses(ZIClassFilter filter)
        throws Exception {
        return classInfo.getClasses(new ZCompoundClassFilter(myClassFilter, filter));
    }

    public Set<String> getClassesAnnotatedWith(String annotationClassName)
        throws Exception {
        return classInfo.getClassesAnnotatedWith(annotationClassName);
    }

    public Set<String> getClassesInstanceof(String className)
        throws Exception {
        return classInfo.getClassesInstanceof(className);
    }

    public List<Set<String>> getCycles()
        throws Exception {
        return ZFilterUtil.filter(classInfo.getCycles(), myClassFilter);
    }

    public ZIClassEdge getEdge(String t1, String t2)
        throws Exception {
        ZIClassEdge ret = classInfo.getEdge(t1, t2);
        if (!myClassEdgeFilter.accept(ret)) {
            return null;
        }
        return ret;
    }

    public Set<ZIClassEdge> getEdges()
        throws Exception {
        return classInfo.getEdges(myClassEdgeFilter);
    }

    public Set<ZIClassEdge> getEdges(ZIClassEdgeFilter edgeFilter)
        throws Exception {
        return classInfo.getEdges(new ZCompoundClassEdgeFilter(myClassEdgeFilter, edgeFilter));
    }

    public Set<ZIClassEdge> getPredecessorEdges(String name, boolean transitive)
        throws Exception {
        return classInfo.getPredecessorEdges(name, myClassEdgeFilter, transitive);
    }

    public Set<ZIClassEdge> getPredecessorEdges(String name, ZIClassEdgeFilter edgeFilter, boolean transitive)
        throws Exception {
        return classInfo.getPredecessorEdges(name, new ZCompoundClassEdgeFilter(myClassEdgeFilter, edgeFilter), transitive);
    }

    public Set<String> getPredecessors(Set<String> classes, boolean transitive)
        throws Exception {
        return classInfo.getPredecessors(classes, myClassEdgeFilter, transitive);
    }

    public Set<String> getPredecessors(String name, boolean transitive)
        throws Exception {
        return classInfo.getPredecessors(name, myClassEdgeFilter, transitive);
    }

    public Set<String> getPredecessors(Set<String> set, ZIClassEdgeFilter edgeFilter, boolean transitive)
        throws Exception {
        return classInfo.getPredecessors(set, new ZCompoundClassEdgeFilter(myClassEdgeFilter, edgeFilter), transitive);
    }

    public Set<String> getPredecessors(String item, ZIClassEdgeFilter edgeFilter, boolean transitive)
        throws Exception {
        return classInfo.getPredecessors(item, new ZCompoundClassEdgeFilter(myClassEdgeFilter, edgeFilter), transitive);
    }

    public Set<String> getScannedClasses()
        throws Exception {
        Set<String> ret = classInfo.getScannedClasses();
        ret = ZFilterUtil.filter(ret, myClassFilter);
        return ret;
    }

    public ZIPath getShortestPath(String v1, String v2)
        throws Exception {
        return classInfo.getShortestPath(v1, v2);
    }

    public Set<ZIClassEdge> getSuccessorEdges(String name, ZIClassEdgeFilter edgeFilter, boolean transitive)
        throws Exception {
        return classInfo.getSuccessorEdges(name, new ZCompoundClassEdgeFilter(myClassEdgeFilter, edgeFilter), transitive);
    }

    public Set<ZIClassEdge> getSuccessorEdges(String name, boolean transitive)
        throws Exception {
        return classInfo.getSuccessorEdges(name, myClassEdgeFilter, transitive);
    }

    public Set<String> getSuccessors(Set<String> classes, boolean transitive)
        throws Exception {
        return classInfo.getSuccessors(classes, myClassEdgeFilter, transitive);
    }

    public Set<String> getSuccessors(String name, boolean transitive)
        throws Exception {
        return classInfo.getSuccessors(name, myClassEdgeFilter, transitive);
    }

    public Set<String> getSuccessors(Set<String> set, ZIClassEdgeFilter edgeFilter, boolean transitive)
        throws Exception {
        return classInfo.getSuccessors(set, new ZCompoundClassEdgeFilter(myClassEdgeFilter, edgeFilter), transitive);
    }

    public Set<String> getSuccessors(String item, ZIClassEdgeFilter edgeFilter, boolean transitive)
        throws Exception {
        return classInfo.getSuccessors(item, new ZCompoundClassEdgeFilter(myClassEdgeFilter, edgeFilter), transitive);
    }

    public Set<String> getSuperclasses(String className)
        throws Exception {
        return classInfo.getSuperclasses(className);
    }

    public boolean isAnnotatedWith(String className, String annotationClassName)
        throws Exception {
        return classInfo.isAnnotatedWith(className, annotationClassName);
    }

    public boolean isInstanceof(String name, String superClassName)
        throws Exception {
        return classInfo.isInstanceof(name, superClassName);
    }

    public boolean isPredecessor(String user, String used, boolean transitive)
        throws Exception {
        return classInfo.isPredecessor(user, used, myClassEdgeFilter, transitive);
    }

    public boolean isPredecessor(String user, String used, ZIClassEdgeFilter edgeFilter, boolean transitive)
        throws Exception {
        return classInfo.isPredecessor(user, used, new ZCompoundClassEdgeFilter(myClassEdgeFilter, edgeFilter), transitive);
    }

    public boolean isSuccessor(String user, String used, boolean transitive)
        throws Exception {
        return classInfo.isSuccessor(user, used, myClassEdgeFilter, transitive);
    }

    public boolean isSuccessor(String user, String used, ZIClassEdgeFilter edgeFilter, boolean transitive)
        throws Exception {
        return classInfo.isSuccessor(user, used, new ZCompoundClassEdgeFilter(myClassEdgeFilter, edgeFilter), transitive);
    }

    public Element toElement()
        throws Exception {
        return classInfo.toElement();
    }

    public String toXML()
        throws Exception {
        return classInfo.toXML();
    }

    public ZIClassInfo getClassInfo() {
        return classInfo;
    }

    public boolean isClass(String name)
        throws Exception {
        return classInfo.isClass(name) && myClassFilter.accept(name);
    }

    public boolean isEdge(String name1, String name2)
        throws Exception {
        if (!classInfo.isEdge(name1, name2)) {
            return false;
        }
        ZIClassEdge edge = classInfo.getEdge(name1, name2);
        return myClassEdgeFilter.accept(edge);
    }

    public Set<ZIClassEdge> getPredecessorEdges(Set<String> set, ZIClassEdgeFilter filter, boolean transitive)
        throws Exception {        
        return classInfo.getPredecessorEdges(set, new ZCompoundClassEdgeFilter(myClassEdgeFilter, filter), transitive);
    }

    public Set<ZIClassEdge> getSuccessorEdges(Set<String> set, ZIClassEdgeFilter filter, boolean transitive)
        throws Exception {
        return classInfo.getSuccessorEdges(set, new ZCompoundClassEdgeFilter(myClassEdgeFilter, filter), transitive);
    }
}
