/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.classinfo;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class ZClassUtils {
    private static final Set<String> primitiveTypes = new HashSet<String>();

    private static final Set<String> primitiveClasses = new HashSet<String>();

    static {
        primitiveTypes.add("void");
        primitiveTypes.add("int");
        primitiveTypes.add("boolean");
        primitiveTypes.add("long");
        primitiveTypes.add("double");
        primitiveTypes.add("float");
        primitiveTypes.add("byte");
        primitiveTypes.add("char");
        primitiveTypes.add("short");

        primitiveClasses.add(Integer.class.getName());
        primitiveClasses.add(Boolean.class.getName());
        primitiveClasses.add(Long.class.getName());
        primitiveClasses.add(Double.class.getName());
        primitiveClasses.add(Float.class.getName());
        primitiveClasses.add(Byte.class.getName());
        primitiveClasses.add(Character.class.getName());
        primitiveClasses.add(Short.class.getName());
    }

    public static String getPackageName(String className) {
        int idx = className.lastIndexOf('.');
        if (idx < 0) {
            return "";
        } else {
            return className.substring(0, idx);
        }
    }

    public static String getSimpleClassName(String className) {
        int idx = className.lastIndexOf('.');
        if (idx < 0) {
            return className;
        } else {
            return className.substring(idx + 1);
        }
    }

    public static String getDeclaringClass(String className) {
        int idx = className.lastIndexOf('$');
        if (idx < 0) {
            return className;
        } else {
            return className.substring(0, idx);
        }
    }

    public static String getNestedClassName(String className) {
        int idx = className.lastIndexOf('$');
        if (idx < 0) {
            return className;
        } else {
            return className.substring(idx + 1);
        }
    }

    public static String getTopClassName(String className) {
        int idx = className.indexOf('$');
        if (idx < 0) {
            return className;
        } else {
            return className.substring(0, idx);
        }
    }

    public static boolean isPrimitiveType(String name) {
        return primitiveTypes.contains(name.toString());
    }

    public static boolean isPrimitiveClass(String name) {
        return primitiveClasses.contains(name.toString());
    }

    public static boolean isPrimitive(String name) {
        return isPrimitiveType(name) || isPrimitiveClass(name);
    }

    public static Set<String> filterPrimitive(Set<String> set)
        throws Exception {
        return filter(set, new ZIClassFilter() {
            public boolean accept(String t)
                throws Exception {
                return !ZClassUtils.isPrimitive(t);
            }
        });
    }

    public static Set<String> filterPrimitiveTypes(Set<String> set)
        throws Exception {
        return filter(set, new ZIClassFilter() {
            public boolean accept(String t)
                throws Exception {
                return !ZClassUtils.isPrimitiveType(t);
            }
        });
    }

    public static Set<String> filterPrimitiveClasses(Set<String> set)
        throws Exception {
        return filter(set, new ZIClassFilter() {
            public boolean accept(String t)
                throws Exception {
                return !ZClassUtils.isPrimitiveClass(t);
            }
        });
    }

    public static Set<String> filter(Set<String> set, ZIClassFilter filter)
        throws Exception {
        if (filter == null) {
            return new HashSet<String>(set);
        }
        Set<String> ret = new HashSet<String>();
        for (String t : set) {
            if (filter.accept(t)) {
                ret.add(t);
            }
        }
        return ret;
    }

    public static boolean isInnerClass(String name) {
        return name.indexOf('$') >= 0;
    }

	public static List<String> getLocationNames(ZIClassInfo classInfo, String name) throws Exception {
		ZIClassMeta meta = classInfo.getClassMeta(name);
		List<ZILocation> locations  = meta.getLocations();
		
		List<String> ret = new ArrayList<String>();
		for(ZILocation loc: locations)
		{
			ret.add(loc.getName());
		}
		return ret;
	}
}
