/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.classinfo;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.jdom.Element;
import org.jdom.Text;
import org.ztest.jdom.ZXMLUtil;

public class ZPath implements ZIPath {
    private final List<ZIClassEdge> edges = new ArrayList<ZIClassEdge>();

    public static final Comparator<ZIClassEdge> EDGE_COMPARATOR = new Comparator<ZIClassEdge>() {

        public int compare(ZIClassEdge e1, ZIClassEdge e2) {
            if (e1.getSource().equals(e2.getTarget())) {
                return 1;
            }
            if (e1.getTarget().equals(e2.getSource())) {
                return -11;
            }
            return 0;
        }
    };

    public ZPath(List<ZIClassEdge> classes) {
        this.edges.addAll(classes);
    }

    public Element toElement()
        throws Exception {
        Element ret = new Element("path");
        for (ZIClassEdge edge : edges) {
            ret.addContent(toElement(edge));
        }
        return ret;
    }

    public String toXML()
        throws Exception {
        return ZXMLUtil.toXML(toElement());
    }

    private Element toElement(ZIClassEdge edge)
        throws Exception {
        Element ret = new Element("edge");
        Element srcNode = new Element("class");
        srcNode.addContent(new Text(String.valueOf(edge.getSource())));
        ret.addContent(srcNode);

        Element tgtNode = new Element("class");
        tgtNode.addContent(new Text(String.valueOf(edge.getTarget())));
        ret.addContent(tgtNode);
        ret.addContent(edge.getMetaData().toElement());
        return ret;
    }

    public List<ZIClassEdge> getEdges() {
        return edges;
    }

    public Set<String> getRelatedClasses() {
        Set<String> ret = new HashSet<String>();
        for (ZIClassEdge e : edges) {
            ret.add(e.getSource());
            ret.add(e.getTarget());
        }
        return ret;
    }

    public Set<ZIClassEdge> getRelatedEdges() {
        Set<ZIClassEdge> ret = new HashSet<ZIClassEdge>(edges);
        return ret;
    }

    public int compareTo(ZIPath o) {
        return getEdges().get(0).getSource().compareTo(o.getEdges().get(0).getSource());
    }

}
