/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.classinfo.impl;

import org.jdom.Element;
import org.ztest.classinfo.ZIClassEdge;
import org.ztest.classinfo.ZIClassEdgeMeta;
import org.ztest.graph.ZIEdge;
import org.ztest.jdom.ZXMLUtil;

class ZClassEdge implements ZIClassEdge {
    private static final long serialVersionUID = 1L;    

    private final ZIEdge<String, ZIClassEdgeMeta> edge;

    public ZClassEdge(ZIEdge<String, ZIClassEdgeMeta> edge) {
        super();
        this.edge = edge;
    }

    public String getSource() {
        return edge.getSource();
    }

    public String getTarget() {
        return edge.getTarget();
    }

    public ZIClassEdgeMeta getMetaData() {
        ZIClassEdgeMeta meta = edge.getMetaData();
        if (meta == null) {
            meta = new ZClassEdgeMeta();
            edge.setMetaData(meta);
        }
        return meta;
    }

    @Override
    public boolean equals(Object obj) {
        ZClassEdge other = (ZClassEdge)obj;
        return edge.equals(other.edge);
    }

    @Override
    public int hashCode() {
        return edge.hashCode();
    }

    public boolean isLoop() {
        return edge.isLoop();
    }

    public int compareTo(ZIClassEdge o) {
        int ret = getSource().compareTo(o.getSource());
        if (ret != 0) {
            return ret;
        }

        return getTarget().compareTo(o.getTarget());
    }

    public void fromElement(Element elem) {
        // TODO Auto-generated method stub

    }

    public Element toElement()
        throws Exception {
        // TODO Auto-generated method stub
        return null;
    }

    public String toXML()
        throws Exception {
        return ZXMLUtil.toXML(toElement());
    }
}
