/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.classinfo.impl;

import java.util.HashSet;
import java.util.Set;

import org.jdom.Element;
import org.ztest.classinfo.ZIClassEdgeMeta;
import org.ztest.classinfo.ZIField;
import org.ztest.classinfo.ZIMethod;
import org.ztest.jdom.ZXMLUtil;

public class ZClassEdgeMeta implements ZIClassEdgeMeta {
    private static final long serialVersionUID = 1L;    

    private boolean usedInExtends;

    private boolean usedInImplements;

    private boolean usedInCode;

    private Set<ZIField> usedInField = new HashSet<ZIField>();

    private Set<ZIMethod> usedInMethod = new HashSet<ZIMethod>();

    public Element toElement()
        throws Exception {
        Element ret = new Element("edge-meta");
        ret.setAttribute("extends", String.valueOf(usedInExtends));
        ret.setAttribute("implements", String.valueOf(usedInImplements));
        ret.setAttribute("usedInCode", String.valueOf(usedInCode));
        for (ZIField m : usedInField) {
            ret.addContent(m.toElement());
        }
        for (ZIMethod m : usedInMethod) {
            ret.addContent(m.toElement());
        }

        return ret;
    }

    public String toXML()
        throws Exception {
        return ZXMLUtil.toXML(toElement());
    }

    public void fromElement(Element elem) {
    }

    public boolean isUsedInExtends() {
        return usedInExtends;
    }

    public void setUsedInExtends(boolean usedInExtends) {
        this.usedInExtends = usedInExtends;
    }

    public boolean isUsedInImplements() {
        return usedInImplements;
    }

    public boolean isInheritance() {
        return usedInImplements || usedInExtends;
    }

    public void setUsedInImplements(boolean usedInImplements) {
        this.usedInImplements = usedInImplements;
    }

    public boolean isUsedInCode() {
        return usedInCode;
    }

    public void setUsedInCode(boolean usedInCode) {
        this.usedInCode = usedInCode;
    }

    public Set<ZIField> getUsedInField() {
        return usedInField;
    }

    public Set<ZIMethod> getUsedInMethod() {
        return usedInMethod;
    }
}
