/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.classinfo.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.jdom.Element;
import org.jdom.Text;
import org.ztest.classinfo.ZIClassMeta;
import org.ztest.classinfo.ZIField;
import org.ztest.classinfo.ZILocation;
import org.ztest.classinfo.ZIMethod;
import org.ztest.jdom.ZXMLUtil;

public class ZClassMeta implements ZIClassMeta {
    private static final long serialVersionUID = 1L;    

    private final List<ZILocation> locations = new ArrayList<ZILocation>();

    private final List<String> annotations = new ArrayList<String>();

    private final List<ZIField> fields = new ArrayList<ZIField>();

    private final List<ZIMethod> methods = new ArrayList<ZIMethod>();

    private final String name;

    private int modifiers;

    public ZClassMeta(String name) {
        super();
        this.name = name;
    }

    public String getName() {
        return name;
    }

    public boolean isScanned() {
        return !locations.isEmpty();
    }

    public int getModifiers() {
        return modifiers;
    }

    public void setModifiers(int modifiers) {
        this.modifiers = modifiers;
    }

    public List<ZILocation> getLocations() {
        return locations;
    }

    public List<String> getAnnotations() {
        return annotations;
    }

    public List<ZIMethod> getMethods() {
        return methods;
    }

    public Element toElement()
        throws Exception {
        Element ret = new Element("class-meta");

        Element modifierNode = new Element("modifiers");
        modifierNode.addContent(new Text(Integer.toBinaryString(modifiers)));
        ret.addContent(modifierNode);

        Collections.sort(locations);
        for (ZILocation loc : locations) {
            Element locationNode = new Element("location");
            locationNode.addContent(loc.toElement());
            ret.addContent(locationNode);
        }

        Collections.sort(annotations);
        for (String ann : annotations) {
            Element annNode = new Element("annotation");
            annNode.addContent(new Text(ann));
            ret.addContent(annNode);
        }

        Collections.sort(fields);
        for (ZIField m : fields) {
            ret.addContent(m.toElement());
        }

        Collections.sort(methods);
        for (ZIMethod m : methods) {
            ret.addContent(m.toElement());
        }
        return ret;
    }

    public String toXML()
        throws Exception {
        return ZXMLUtil.toXML(toElement());
    }

    public List<ZIField> getFields() {
        return fields;
    }

}
