/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.classinfo.impl;

import org.jdom.Element;
import org.ztest.classinfo.ZIField;
import org.ztest.classinfo.ZIMethod;
import org.ztest.jdom.ZXMLUtil;

public class ZField implements ZIField {
    private final String name;

    private final int modifiers;

    public ZField(String name, int modifier) {
        super();
        this.name = name;
        this.modifiers = modifier;
    }

    public Element toElement() {
        Element ret = new Element("field");
        ret.setAttribute("name", name);
        ret.setAttribute("modifiers", Integer.toString(modifiers, 2));
        return ret;
    }

    public String toString() {
        return toXML();
    }

    public String toXML() {
        return ZXMLUtil.toXML(toElement());
    }

    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof ZIMethod)) {
            return false;
        }

        ZIMethod other = (ZIMethod)obj;

        return name.equals(other.getName()) && modifiers == other.getModifiers();
    }

    @Override
    public int hashCode() {
        return name.hashCode();
    }

    public int compareTo(ZIField o) {
        int ret = name.compareTo(o.getName());
        if (ret != 0) {
            return ret;
        }

        return modifiers < o.getModifiers() ? 1 : modifiers == o.getModifiers() ? 0 : -1;
    }

    public String getName() {
        return name;
    }

    public int getModifiers() {
        return modifiers;
    }

}
