/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.graph;

import java.util.List;
import java.util.Map;
import java.util.Set;

import org.ztest.jdom.ZWriteXML;

public interface ZIGraph<T, EM extends ZWriteXML, NM extends ZWriteXML> extends ZWriteXML {
    public void addNode(T name);

    public ZIEdge<T, EM> addEdge(T name1, T name2);

    public ZIEdge<T, EM> getEdge(T t1, T t2);

    public Set<T> getNodeSet();

    public Set<T> getNodeSet(ZINodeFilter<T> filter) throws Exception;

    public Set<ZIEdge<T, EM>> getEdgeSet();

    public Set<ZIEdge<T, EM>> getEdgeSet(ZIEdgeFilter<T, EM> filter)
        throws Exception;

    public Map<String, NM> getNodeMeta(T n);

    public boolean isNode(T name);

    public boolean isEdge(T user, T used);

    /**
     *
     * @return the real cycles in the graph without selfcycles (cycles of length
     *         1)
     */
    public List<Set<T>> getCycles();

    public List<Set<T>> getCycles(boolean includeSelfcycles);

    public List<ZIEdge<T, EM>> getShortestPath(T t1, T t2);

    public Set<T> getSuccessors(Set<T> itemSet, ZIEdgeFilter<T, EM> filter, boolean transitive)
        throws Exception;

    public Set<T> getSuccessors(T item, ZIEdgeFilter<T, EM> filter, boolean transitive)
        throws Exception;

    public boolean isSuccessor(T item, T successor, ZIEdgeFilter<T, EM> filter, boolean transitive)
        throws Exception;

    public Set<ZIEdge<T, EM>> getSuccessorEdges(T item, ZIEdgeFilter<T, EM> filter, boolean transitive)
        throws Exception;

    public Set<ZIEdge<T, EM>> getSuccessorEdges(Set<T> itemSet, ZIEdgeFilter<T, EM> filter, boolean transitive)
        throws Exception;

    public Set<T> getPredecessors(T item, ZIEdgeFilter<T, EM> filter, boolean transitive)
        throws Exception;

    public Set<T> getPredecessors(Set<T> itemSet, ZIEdgeFilter<T, EM> filter, boolean transitive)
        throws Exception;

    public Set<ZIEdge<T, EM>> getPredecessorEdges(T item, ZIEdgeFilter<T, EM> filter, boolean transitive)
        throws Exception;

    public Set<ZIEdge<T, EM>> getPredecessorEdges(Set<T> itemSet, ZIEdgeFilter<T, EM> filter, boolean transitive)
        throws Exception;

    public boolean isPredecessor(T item, T predecessor, ZIEdgeFilter<T, EM> filter, boolean transitive)
        throws Exception;
}
