/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.graph.jgrapht;

import org.jdom.Element;
import org.ztest.graph.ZIEdge;
import org.ztest.jdom.ZWriteXML;
import org.ztest.jdom.ZXMLUtil;

class ZEdge<N, EM extends ZWriteXML> implements ZIEdge<N, EM> {
    private final N source;

    private final N target;

    private EM metaData;

    public ZEdge(N source, N target) {
        super();
        this.source = source;
        this.target = target;
    }

    public N getSource() {
        return source;
    }

    public N getTarget() {
        return target;
    }

    public EM getMetaData() {
        return metaData;
    }

    @Override
    public boolean equals(Object obj) {
        ZEdge<N, EM> other = (ZEdge<N, EM>)obj;
        return source.equals(other.source) && target.equals(other.target);
    }

    @Override
    public int hashCode() {
        return source.hashCode() + target.hashCode();
    }

    public Element toElement()
        throws Exception {
        Element ret = new Element("edge");

        Element srcNode = new Element("node");
        srcNode.setAttribute("name", String.valueOf(source));
        ret.addContent(srcNode);

        Element tgtNode = new Element("node");
        tgtNode.setAttribute("name", String.valueOf(target));
        ret.addContent(tgtNode);

        if (metaData != null) {
            ret.addContent(metaData.toElement());
        }

        return ret;
    }

    public String toXML()
        throws Exception {
        return ZXMLUtil.toXML(toElement());
    }

    public void setMetaData(EM meta) {
        this.metaData = meta;
    }

    public boolean isLoop() {
        return source.equals(target);
    }
}
