/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.jdom;

import java.io.IOException;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

public class ZXMLUtil {
    static final Logger log = Logger.getLogger(ZXMLUtil.class);

    public static <T extends Comparable<T>> List<T> sort(Set<T> items) {
        List<T> ret = new ArrayList<T>(items.size());
        ret.addAll(items);
        Collections.sort(ret);
        return ret;
    }

    public static String toXML(Element root) {
        root.setAttribute("home", "http://sourceforge.net/projects/ztest");
        root.setAttribute("author", "http://www.gerdziegler.de");
        Document doc = new Document(root);
        StringWriter sw = new StringWriter();
        XMLOutputter outputter = new XMLOutputter(Format.getPrettyFormat().setExpandEmptyElements(false));
        try {
            outputter.output(doc, sw);
            return sw.toString();
        } catch (IOException e) {
            log.error("", e);
            return e.getMessage();
        }
    }
}
