/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.output.classinfo;

import java.lang.reflect.Modifier;

import org.ztest.classinfo.ZIClassEdge;
import org.ztest.classinfo.ZIClassEdgeMeta;
import org.ztest.classinfo.ZIClassInfo;
import org.ztest.classinfo.ZIClassMeta;
import org.ztest.classinfo.ZIField;
import org.ztest.classinfo.ZIMethod;
import org.ztest.output.graphml.ZIGraphMLDecorator;
import org.ztest.output.graphml.ZIGraphMLEdge;
import org.ztest.output.graphml.ZIGraphMLGroupNode;
import org.ztest.output.graphml.ZIGraphMLNode;
import org.ztest.output.graphml.style.ZEdgeStyle;
import org.ztest.output.graphml.style.ZGroupNodeStyle;
import org.ztest.output.graphml.style.ZLineStyle;
import org.ztest.output.graphml.style.ZNodeStyle;

public class ZClassInfoDecorator implements ZIGraphMLDecorator {
    private final ZIClassInfo classInfo;

    public ZClassInfoDecorator(ZIClassInfo classInfo) {
        super();
        this.classInfo = classInfo;
    }

    public ZNodeStyle getStyle(ZIGraphMLNode node, ZNodeStyle style)
        throws Exception {
        style.getNodeLabelStyle().setText(node.getId());

        ZIClassMeta meta = classInfo.getClassMeta(node.getId());

        if (Modifier.isInterface(meta.getModifiers())) {
            style.getBorderStyle().getLineStyle().setType(ZLineStyle.LINE_TYPE_DASHED);
            style.getFillStyle().setColors("#CCDDFF");
        }

        return style;
    }

    public ZGroupNodeStyle getStyle(ZIGraphMLGroupNode group, ZGroupNodeStyle style)
        throws Exception {
        style.getNodeLabelStyle().setText(group.getId());
        return style;
    }

    private static String cut(String s){
        return cut(s, 30);
    }
    private static String cut(String s, int len){
        if(s.length()>=len){
           s = s.substring(s.length()-len); 
        }
        return s;
    }
    
    public ZEdgeStyle getStyle(ZIGraphMLEdge edge, ZEdgeStyle style)
        throws Exception {

        StringBuffer txt = new StringBuffer();
        ZIClassEdge classEdge = classInfo.getEdge(edge.getSourceId(), edge.getTargetId());
        ZIClassEdgeMeta meta = classEdge.getMetaData();
        for (ZIField f : meta.getUsedInField()) {
            txt.append(f.getName());
            txt.append(' ');
        }
        for (ZIMethod f : meta.getUsedInMethod()) {
            txt.append(f.getName());
            txt.append(' ');
        }
        
        if(meta.isUsedInCode()) {
            txt.append("used in code");
            txt.append(' ');
        }

        txt.append("[");
        txt.append(cut(edge.getSourceId()));
        txt.append("] --> [");
        txt.append(cut(edge.getTargetId()));
        txt.append("]");

        style.setText(txt.toString());
        
        boolean inh = meta.isInheritance();
        if (inh) {
            style.setArrow2(ZEdgeStyle.ARROW_TYPE_WHITE_DELTA);
        } else {
            style.setArrow2(ZEdgeStyle.ARROW_TYPE_STANDARD);
        }

        if (meta.isUsedInImplements()) {
            style.getLineStyle().setType(ZLineStyle.LINE_TYPE_DASHED);
        } else {
            style.getLineStyle().setType(ZLineStyle.LINE_TYPE_LINE);
        }

        return style;
    }
}
