/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.sets;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import org.jdom.Element;
import org.jdom.Text;
import org.ztest.jdom.ZXMLUtil;

/**
 * Named set of classes that holds the classes to be tested.
 *
 * @author gerdziegler.de
 *
 */
public abstract class ZAbstractClassSet implements ZIClassSet {
    private final String name;

    public ZAbstractClassSet(String name) {
        if (name == null) {
            throw new RuntimeException("name cannot be null");
        }
        this.name = name;
    }

    public String getName() {
        return name;
    }

    public int compareTo(ZIClassSet l) {
        return name.compareTo(l.getName());
    }

    @Override
    public String toString() {
        return name;
    }

    public String toXML()
        throws Exception {
        return ZXMLUtil.toXML(toElement());
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof ZAbstractClassSet && name.equals(((ZAbstractClassSet)obj).name);
    }

    @Override
    public int hashCode() {
        return name.hashCode();
    }

    public Element toElement()
        throws Exception {
        return toElement(true);
    }

    public Element toElementNoContent()
        throws Exception {
        return toElement(false);
    }

    protected Element toElement(boolean showContent)
        throws Exception {
        Element root = new Element("class-set");
        root.setAttribute("name", name);

        Set<String> classes = getClasses();
        root.setAttribute("count", String.valueOf(classes.size()));
        List<String> sorted = sort(classes);
        for (String s : sorted) {
            Element classNode = new Element("class");
            classNode.addContent(new Text(s));
            root.addContent(classNode);
        }
        return root;
    }

    protected static <T extends Comparable<T>> List<T> sort(Set<T> items) {
        List<T> ret = new ArrayList<T>(items.size());
        ret.addAll(items);
        Collections.sort(ret);
        return ret;
    }
}
