/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.sets;

import java.lang.reflect.Modifier;
import java.util.Set;

import org.ztest.classinfo.ZIClassEdge;
import org.ztest.classinfo.ZIClassEdgeFilter;
import org.ztest.classinfo.ZIClassInfo;
import org.ztest.classinfo.ZIField;

/**
 * Recursively detects the field-types that should be serializable. Stops at
 * interfaces. Maybe should also scan implementors of interfaces.
 *
 * @author gerdziegler.de
 *
 */
public class ZFieldTypesToSerializeSet extends ZClassSet {
    protected final ZIClassInfo classInfo;

    private final ZIClassSet roots;

    public ZFieldTypesToSerializeSet(ZIClassInfo classInfo, ZIClassSet roots)
        throws Exception {
        super("field-types-to-serialize-" + roots.getName());
        this.classInfo = classInfo;
        this.roots = roots;
        initClasses();
    }

    protected void initClasses()
        throws Exception {
        Set<String> memberSet = classInfo.getSuccessors(roots.getClasses(), new ZIClassEdgeFilter() {
            public boolean accept(ZIClassEdge edge) {
                for (ZIField fm : edge.getMetaData().getUsedInField()) {
                    if (!Modifier.isStatic(fm.getModifiers()) && !Modifier.isTransient(fm.getModifiers())) {
                        return true;
                    }
                }
                return false;
            }
        }, true);
        getClasses().addAll(roots.getClasses());
        getClasses().addAll(memberSet);
    }

    public ZIClassSet getRoots() {
        return roots;
    }
}
