/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.sets.ejb3;

import org.ztest.classinfo.ZIClassInfo;

public class ZEJB3Util {
    public static boolean isSessionBean(ZIClassInfo classInfo, String name)
        throws Exception {
        return isStatelessSessionBean(classInfo, name) || isStatefulSessionBean(classInfo, name);
    }

    public static boolean isStatelessSessionBean(ZIClassInfo classInfo, String name)
        throws Exception {
        return classInfo.isAnnotatedWith(name, "javax.ejb.Stateless");
    }

    public static boolean isStatefulSessionBean(ZIClassInfo classInfo, String name)
        throws Exception {
        return classInfo.isAnnotatedWith(name, "javax.ejb.Stateful");
    }

    public static boolean isSessionBeanInterface(ZIClassInfo classInfo, String name)
        throws Exception {
        return isLocalSessionBeanInterface(classInfo, name) || isRemoteSessionBeanInterface(classInfo, name);
    }

    public static boolean isLocalSessionBeanInterface(ZIClassInfo classInfo, String name)
        throws Exception {
        return classInfo.isAnnotatedWith(name, "javax.ejb.Local") && !isSessionBean(classInfo, name);
    }

    public static boolean isRemoteSessionBeanInterface(ZIClassInfo classInfo, String name)
        throws Exception {
        return classInfo.isAnnotatedWith(name, "javax.ejb.Remote") && !isSessionBean(classInfo, name);
    }

    public static boolean isEntityBean(ZIClassInfo classInfo, String name)
        throws Exception {
        return classInfo.isAnnotatedWith(name, "javax.persistence.Entity");
    }
}
