/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.tests;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import org.jdom.Element;
import org.ztest.classinfo.ZClassUtils;
import org.ztest.classinfo.ZIClassFilter;
import org.ztest.classinfo.ZIClassInfo;
import org.ztest.jdom.ZXMLUtil;
import org.ztest.sets.ZIClassSet;

public abstract class ZClassSetTest<F extends ZITestFailure> implements ZIClassSetTest, ZITest {
    private final ZIClassInfo classInfo;

    private final String name;

    private final ZIClassSet classSet;

    private ZIClassFilter filter;

    private final List<F> failures = new ArrayList<F>();

    public ZClassSetTest(String name, ZIClassInfo classInfo, ZIClassSet classSet) {
        super();
        this.name = name;
        this.classInfo = classInfo;
        this.classSet = classSet;
    }

    public String toXML()
        throws Exception {
        return ZXMLUtil.toXML(toElement());
    }

    public Element toElement()
        throws Exception {
        Element ret = new Element(name);
        ret.setAttribute("class-set", classSet.getName());
        ret.addContent(ZTestUtil.toElement(failures));
        return ret;
    }

    public List<F> getFailures() {
        return failures;
    }

    public ZIClassInfo getClassInfo() {
        return classInfo;
    }

    public Set<String> getFilteredClasses()
        throws Exception {
        return ZClassUtils.filter(classSet.getClasses(), filter);
    }

    public List<String> getFilteredClassesSorted()
        throws Exception {
        List<String> ret = new ArrayList<String>(getFilteredClasses());
        Collections.sort(ret);
        return ret;
    }

    public ZIClassFilter getFilter() {
        return filter;
    }

    public void setFilter(ZIClassFilter filter) {
        this.filter = filter;
    }

    public String getName() {
        return name;
    }

    public ZIClassSet getClassSet() {
        return classSet;
    }
}
