/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.tests;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.HashSet;
import java.util.Set;

import org.jdom.CDATA;
import org.jdom.Element;
import org.ztest.classinfo.ZIClassEdge;
import org.ztest.jdom.ZXMLUtil;

public class ZCompoundFailure implements ZITestFailure {
    private final ZITest test;

    private final Exception exception;

    public ZCompoundFailure(final ZITest test) {
        this.test = test;
        this.exception = null;
    }

    public ZCompoundFailure(final ZITest test, Exception exception) {
        this.test = test;
        this.exception = exception;
    }

    public Element toElement()
        throws Exception {
        Element ret = test.toElement();

        if (exception != null) {
            Element ex = new Element("exception");
            ret.addContent(ex);
            StringWriter w = new StringWriter();
            exception.printStackTrace(new PrintWriter(w));
            CDATA cdata = new CDATA(w.toString());
            ex.addContent(cdata);
        }

        return ret;
    }

    public String toXML()
        throws Exception {
        return ZXMLUtil.toXML(toElement());
    }

    public int compareTo(ZITestFailure o) {
        return toString().compareTo(o.toString());
    }

    public Set<String> getRelatedClasses()
        throws Exception {
        Set<String> ret = new HashSet<String>();
        for (ZITestFailure f : test.getFailures()) {
            ret.addAll(f.getRelatedClasses());
        }
        return ret;
    }

    public Set<ZIClassEdge> getRelatedEdges()
        throws Exception {
        Set<ZIClassEdge> ret = new HashSet<ZIClassEdge>();
        for (ZITestFailure f : test.getFailures()) {
            ret.addAll(f.getRelatedEdges());
        }
        return ret;
    }
}
