/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.tests;

import java.util.ArrayList;
import java.util.List;

import org.jdom.Element;
import org.ztest.jdom.ZXMLUtil;

public class ZCompoundTest implements ZITest {
    private final List<ZCompoundFailure> failures = new ArrayList<ZCompoundFailure>();

    private final List<ZITest> tests = new ArrayList<ZITest>();

    public List<ZITest> getTests() {
        return tests;
    }

    public void run()
        throws Exception {
        for (ZITest test : tests) {
            try {
                test.run();
                if (!test.getFailures().isEmpty()) {
                    failures.add(new ZCompoundFailure(test));
                }
            } catch (Exception e) {
                failures.add(new ZCompoundFailure(test, e));
            }
        }
    }

    public String toXML()
        throws Exception {
        return ZXMLUtil.toXML(toElement());
    }

    public Element toElement()
        throws Exception {
        Element ret = new Element("compound-test");
        Element failuresNode = ZTestUtil.toElement(failures);
        failuresNode.setAttribute("total", String.valueOf(computeNestedFailures(this)));
        ret.addContent(failuresNode);
        return ret;
    }

    private int computeNestedFailures(ZITest t) {
        int ret = 0;

        if (t instanceof ZCompoundTest) {
            ZCompoundTest ct = (ZCompoundTest)t;
            for (ZITest crt : ct.tests) {
                ret += computeNestedFailures(crt);
            }
        } else {
            ret += t.getFailures().size();
        }

        return ret;
    }

    public List<ZCompoundFailure> getFailures() {
        return failures;
    }
}
