/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.tests;

import java.util.Set;

import org.jdom.Element;
import org.jdom.Text;
import org.ztest.classinfo.ZIClassEdge;
import org.ztest.classinfo.ZIClassEdgeFilter;
import org.ztest.classinfo.ZIClassInfo;
import org.ztest.jdom.ZXMLUtil;

public class ZCycleFailure implements ZITestFailure {
    private final Set<String> cycle;

    private final ZIClassInfo classInfo;

    public ZCycleFailure(ZIClassInfo classInfo, Set<String> cycle) {
        this.classInfo = classInfo;
        this.cycle = cycle;
    }

    public Element toElement() {
        Element ret = new Element("cycle-failure");
        // cycleNode.setAttribute("allowed", String.valueOf(allow));
        for (String s : cycle) {
            Element classNode = new Element("class");
            classNode.addContent(new Text(s.toString()));
            ret.addContent(classNode);
        }
        return ret;
    }

    public String toString() {
        return toXML();
    }

    public String toXML() {
        return ZXMLUtil.toXML(toElement());
    }

    public Set<String> getRelatedClasses() {
        return cycle;
    }

    public Set<ZIClassEdge> getRelatedEdges()
        throws Exception {
        return classInfo.getEdges(new ZIClassEdgeFilter() {
            public boolean accept(ZIClassEdge edge) {
                return !edge.isLoop() && cycle.contains(edge.getSource()) && cycle.contains(edge.getTarget());
            }
        });
    }
}
