/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.tests;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.jdom.Element;
import org.ztest.classinfo.ZIClassInfo;
import org.ztest.jdom.ZXMLUtil;

public class ZCycleTest implements ZITest {
    private final ZIClassInfo classInfo;

    private final List<ZCycleFailure> failures = new ArrayList<ZCycleFailure>();

    private List<Set<String>> cycles;

    private final List<Set<String>> allowed = new ArrayList<Set<String>>();

    private boolean allowInnerClassCycles = false;

    public boolean isAllowInnerClassCycles() {
	return allowInnerClassCycles;
    }

    public void setAllowInnerClassCycles(boolean allowInnerClassCycles) {
	this.allowInnerClassCycles = allowInnerClassCycles;
    }

    public ZCycleTest(ZIClassInfo classInfo) {
	this.classInfo = classInfo;
    }

    public void run() throws Exception {
	List<Set<String>> allCycles = classInfo.getCycles();
	for (Set<String> cycle : allCycles) {
	    if (!isAllowed(cycle)) {
		failures.add(new ZCycleFailure(classInfo, cycle));
	    }
	}
    }

    public boolean isValid() {
	if (cycles.isEmpty()) {
	    return true;
	}
	if (allowed.isEmpty()) {
	    return false;
	}

	for (Set<String> c : cycles) {
	    if (!isAllowed(c)) {
		return false;
	    }
	}
	return true;
    }

    public String toXML() throws Exception {
	return ZXMLUtil.toXML(toElement());
    }

    public Element toElement() throws Exception {
	Element ret = new Element("cycle-test");
	ret.addContent(ZTestUtil.toElement(failures));
	return ret;
    }

    public List<Set<String>> getAllowed() {
	return allowed;
    }

    public boolean isAllowed(Set<String> c) {
	for (Set<String> a : allowed) {
	    if (a.containsAll(c)) {
		return true;
	    }
	}
	if (allowInnerClassCycles) {
	    String outerClass = null;
	    for (String s : c) {
		int idx = s.indexOf('$');
		if (idx >= 0) {
		    s = s.substring(0, idx);
		}
		if (outerClass == null) {
		    outerClass = s;
		} else if (!s.equals(outerClass)) {
		    return false;
		}
	    }
	    return true;
	}
	return false;
    }

    public void allow(String... items) {
	Set<String> set = new HashSet<String>();
	for (String s : items) {
	    set.add(s);
	}
	allowed.add(set);
    }

    public List<ZCycleFailure> getFailures() {
	return failures;
    }
}
