/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.tests;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.jdom.Element;
import org.ztest.classinfo.ZIClassEdge;
import org.ztest.classinfo.ZIClassInfo;
import org.ztest.classinfo.ZIPath;
import org.ztest.jdom.ZXMLUtil;

public class ZDisallowUsageFailure implements ZITestFailure {
    private final ZIClassInfo classInfo;

    private Map<String, Set<String>> usage = new HashMap<String, Set<String>>();

    private Set<String> relatedClasses = new HashSet<String>();

    public ZDisallowUsageFailure(final ZIClassInfo classInfo) {
        this.classInfo = classInfo;
    }

    public Map<String, Set<String>> getUsage() {
        return usage;
    }

    public void add(String s1, String s2) {
        Set<String> l = usage.get(s1);
        if (l == null) {
            l = new HashSet<String>();
            usage.put(s1, l);
        }
        l.add(s2);
    }

    public Element toElement()
        throws Exception {
        Element root = new Element("usage-infos");
        root.setAttribute("count", String.valueOf(getUsage().size()));
        for (Map.Entry<String, Set<String>> en : getUsage().entrySet()) {
            String name1 = en.getKey();

            Element usesNode = new Element("usage");
            usesNode.setAttribute("class", name1.toString());

            for (String name2 : en.getValue()) {
                ZIPath path = classInfo.getShortestPath(name1, name2);
                usesNode.addContent(path.toElement());
                relatedClasses.addAll(path.getRelatedClasses());
            }

            root.addContent(usesNode);
        }
        return root;
    }

    public String toXML()
        throws Exception {
        return ZXMLUtil.toXML(toElement());
    }

    public Set<String> getRelatedClasses() {
        return relatedClasses;
    }

    public Set<ZIClassEdge> getRelatedEdges() {
        // TODO Auto-generated method stub
        return null;
    }
}
