/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.tests;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.jdom.Element;
import org.ztest.jdom.ZXMLUtil;
import org.ztest.sets.ZIClassSet;

public class ZDisjunctTest implements ZITest
{
  private final List<ZDisjunctFailure> failures = new ArrayList<ZDisjunctFailure>();

  private final ZIClassSet set1;

  private final ZIClassSet set2;


  public ZDisjunctTest(final ZIClassSet set1, final ZIClassSet set2)
  {
    super();
    this.set1 = set1;
    this.set2 = set2;
  }


  public void run() throws Exception
  {
    Set<String> intersect = new HashSet<String>(set1.getClasses());
    intersect.retainAll(set2.getClasses());
    for (String s : intersect)
    {
      getFailures().add(new ZDisjunctFailure(s));
    }
  }


  public List<ZDisjunctFailure> getFailures()
  {
    return failures;
  }


  public String toXML() throws Exception
  {
    return ZXMLUtil.toXML(toElement());
  }


  public Element toElement() throws Exception
  {
    Element ret = new Element("disjunct-test");
    ret.setAttribute("class-set1", set1.getName());
    ret.setAttribute("class-set2", set2.getName());
    ret.addContent(ZTestUtil.toElement(failures));
    return ret;
  }



}
