/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.tests;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.jdom.Element;
import org.jdom.Text;
import org.ztest.classinfo.ZIClassEdge;
import org.ztest.classinfo.ZIClassInfo;
import org.ztest.classinfo.ZILocation;
import org.ztest.jdom.ZXMLUtil;

public class ZDuplicateClassLocationsFailure implements ZITestFailure {
    private final ZIClassInfo classInfo;

    private final String name;

    public ZDuplicateClassLocationsFailure(ZIClassInfo classInfo, String name) {
        this.classInfo = classInfo;
        this.name = name;
    }

    public Element toElement()
        throws Exception {
        Element ret = new Element("duplicate-class-location-failure");
        Element classNode = new Element("class");
        ret.addContent(classNode);
        classNode.addContent(new Text(name));
        List<ZILocation> locations = new ArrayList<ZILocation>(classInfo.getClassMeta(name).getLocations());
        Collections.sort(locations);
        for (ZILocation loc : locations) {
            ret.addContent(loc.toElement());
        }
        return ret;
    }

    public String toXML()
        throws Exception {
        return ZXMLUtil.toXML(toElement());
    }

    public int compareTo(ZITestFailure tf) {
        if (!(tf instanceof ZDuplicateClassLocationsFailure)) {
            return -1;
        }
        ZDuplicateClassLocationsFailure v = (ZDuplicateClassLocationsFailure)tf;
        return name.compareTo(v.name);
    }

    public Set<String> getRelatedClasses() {
        Set<String> ret = new HashSet<String>();
        ret.add(name);
        return ret;
    }

    public Set<ZIClassEdge> getRelatedEdges() {
        Set<ZIClassEdge> ret = new HashSet<ZIClassEdge>();
        return ret;
    }
}
