/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.tests;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.ztest.classinfo.ZIClassFilter;
import org.ztest.classinfo.ZIClassInfo;

/**
 * Tests for duplicate locations
 * 
 * @author www.gerdziegler.de
 * 
 */
public class ZDuplicateClassLocationsTest extends ZTest<ZDuplicateClassLocationsFailure> {
    static final Logger log = Logger.getLogger(ZDuplicateClassLocationsTest.class);

    public ZDuplicateClassLocationsTest(String name, ZIClassInfo classInfo) {
        super(name, classInfo);
    }

    public ZDuplicateClassLocationsTest(ZIClassInfo classInfo) {
        this("duplicate-class-locations-test", classInfo);
    }

    public void run()
        throws Exception {
        Set<String> duplicates = getClassInfo().getClasses(new ZIClassFilter() {
            public boolean accept(String name)
                throws Exception {
                return getClassInfo().getClassMeta(name).getLocations().size() > 1;
            }
        });

        List<String> dup = new ArrayList<String>(duplicates);
        Collections.sort(dup);
        for (String name : dup) {
            getFailures().add(new ZDuplicateClassLocationsFailure(getClassInfo(), name));
        }
    }
}
