/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.tests;

import java.util.HashSet;
import java.util.Set;

import org.jdom.Element;
import org.jdom.Text;
import org.ztest.classinfo.ZIClassEdge;
import org.ztest.classinfo.ZIClassInfo;
import org.ztest.jdom.ZXMLUtil;

public class ZInstanceofFailure implements ZITestFailure {
    private final ZIClassInfo classInfo;

    private final String className;

    private final String superClassName;

    public ZInstanceofFailure(ZIClassInfo classInfo, String className, String superClassName) {
        this.classInfo = classInfo;
        this.className = className;
        this.superClassName = superClassName;
    }

    public Element toElement()
        throws Exception {
        Element ret = new Element("instanceof-failure");
        ret.setAttribute("should-be", superClassName);
        Element classNode = new Element("class");
        ret.addContent(classNode);
        classNode.addContent(new Text(className));

        Set<String> implemented = classInfo.getSuperclasses(className);
        if (!implemented.isEmpty()) {
            Element instanceofNode = new Element("instanceof");
            ret.addContent(instanceofNode);
            for (String base : implemented) {
                Element baseClassNode = new Element("class");
                baseClassNode.addContent(new Text(base.toString()));
                instanceofNode.addContent(baseClassNode);
            }
        }

        return ret;
    }

    public String toXML()
        throws Exception {
        return ZXMLUtil.toXML(toElement());
    }

    public Set<String> getRelatedClasses() {
        Set<String> ret = new HashSet<String>();
        ret.add(className);
        return ret;
    }

    public Set<ZIClassEdge> getRelatedEdges() {
        Set<ZIClassEdge> ret = new HashSet<ZIClassEdge>();
        return ret;
    }
}
