/*
 * Copyright 2008 Gerd Ziegler (www.gerdziegler.de) Licensed under the Apache
 * License, Version 2.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License. @author
 * www.gerdziegler.de
 */
package org.ztest.tests;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.ztest.classinfo.ZClassUtils;
import org.ztest.classinfo.ZIClassInfo;
import org.ztest.sets.ZIClassSet;

public class ZInstanceofTest extends ZClassSetTest<ZInstanceofFailure> {
    static final Logger log = Logger.getLogger(ZInstanceofTest.class);

    private final String superClassName;

    public ZInstanceofTest(String name, ZIClassInfo classInfo, ZIClassSet classes, String superClassName) {
        super(name, classInfo, classes);
        this.superClassName = superClassName;
    }

    public ZInstanceofTest(ZIClassInfo classInfo, ZIClassSet classes, String superClassName) {
        this("instanceof-test", classInfo, classes, superClassName);
    }

    public void run()
        throws Exception {
        Set<String> items = getFilteredClasses();
        Set<String> implementors = getClassInfo().getClassesInstanceof(superClassName);
        items.removeAll(implementors);
        items = ZClassUtils.filterPrimitiveTypes(items);

        List<String> sorted = new ArrayList<String>(items);
        Collections.sort(sorted);
        for (String s : sorted) {
            getFailures().add(new ZInstanceofFailure(getClassInfo(), s, superClassName));
        }
    }

    public String getSuperClassName() {
        return superClassName;
    }
}
